#include <Servo.h>
#include <LiquidCrystal.h>
#include <DHT.h>

//---------------------------------------------------------------------------
//------------------------------Initilization--------------------------------
// Pin definitions
const int ldrPin = A1;          // Pin where the LDR is connected
const int servoPin = 6;         // Pin where the servo motor is connected
const int rs = 7, en = 8, d4 = 9, d5 = 10, d6 = 11, d7 = 12; // LCD pins
const int button1Pin = 41;       // Pin for Button1 (manual mode)
const int button2Pin = 42;       // Pin for Button2 (automatic mode)
const int potPin = A0;          // Pin for potentiometer
const int dhtPin = 4;           // Pin for DHT11 data pin
//---------------------------------------------------------------------------
//---------------------------------------------------------------------------


// Debounce
const int LED1 = 31; // green = manual
const int LED2 = 32; // red = processing
const int LED3 = 33; // blue = scanning

boolean lastButton1 = LOW;
boolean currentButton1 = LOW;
boolean ledOn1 = true;
boolean lastButton2 = LOW;
boolean currentButton2 = LOW;
boolean ledOn2 = false;

boolean ledOn3 = false;

// Servo and LCD objects
Servo panelServo;
LiquidCrystal lcd(rs, en, d4, d5, d6, d7);
DHT dht(dhtPin, DHT11);

const int scanRange = 180;      // Range of motion for the LDR scan
const int stepSize = 10;        // Step size for scanning

int ldrValue;
int maxLdrValue = 0;
int bestServoPos = 90;
bool manualMode = true;
bool useProcessing = false;

float lastTemperature = 0.0;
float lastLightIntensity = 0.0;
const float tempThreshold = 0.2;  // Temperature change threshold
const float lightThreshold = 50;  // Light intensity change threshold

boolean debounce(boolean last, int pin) {
  boolean current = digitalRead(pin);
  if (last != current) {
    delay(5);
    current = digitalRead(pin);
  }
  return current;
}


//---------------------------------------------------------------------------
//------------------------------Initilization Con't--------------------------
void setup() {
  Serial.begin(9600);
  panelServo.attach(servoPin);
  panelServo.write(90); // Start at the middle position (90 degrees)
  
  // Initialize the LCD
  lcd.begin(16, 2);
  lcd.print("Initializing...");

  // Initialize button pins as inputs with internal pull-up resistors
  pinMode(button1Pin, INPUT_PULLUP);
  pinMode(button2Pin, INPUT_PULLUP);

  pinMode(LED1, OUTPUT);
  digitalWrite(LED1, HIGH);
  pinMode(LED2, OUTPUT);
  digitalWrite(LED2, HIGH);
  pinMode(LED3, OUTPUT);
  digitalWrite(LED3, HIGH);

  // Initialize DHT11 sensor
  dht.begin();

  lcd.setCursor(0, 1);
  lcd.print("Hello");
}
//---------------------------------------------------------------------------
//---------------------------------------------------------------------------
//---------------------------------------------------------------------------


void loop() {
  // Read the current state of the buttons
  currentButton1 = debounce(lastButton1, button1Pin);
  if (lastButton1 == LOW && currentButton1 == HIGH) {
    ledOn3 = false;
    ledOn1 = !ledOn1;
    ledOn2 = !ledOn2;
    manualMode = !manualMode;
    useProcessing = !useProcessing;
  }
  lastButton1 = currentButton1;
  digitalWrite(LED1, ledOn1);

  currentButton2 = debounce(lastButton2, button2Pin);
  if (lastButton2 == LOW && currentButton2 == HIGH) {
    ledOn3 = false;
    ledOn2 = false;
    ledOn1 = true;
    manualMode = true;
  }
  lastButton2 = currentButton2;
  digitalWrite(LED2, ledOn2);

  // Check for serial input from Processing
  if (Serial.available() > 0) {
    int incomingByte = Serial.read();
    if (incomingByte == 'P') {
      useProcessing = true;
      manualMode = false;
      ledOn2 = true;
      ledOn1 = false;
      ledOn3 = false;
    } else if (incomingByte == 'A') {
      useProcessing = false;
      manualMode = false;
      ledOn3 = true;
      ledOn2 = false;
      ledOn1 = false;
    } else if (useProcessing && incomingByte >= 0 && incomingByte <= 180) {
      panelServo.write(incomingByte);
    }
  }
  digitalWrite(LED3, ledOn3);

  //--------------------------------------------------------------------------
  //------------------------------Manual Mode--------------------------------
  if (manualMode) {
    // Manual mode: control servo with potentiometer
    int potValue = analogRead(potPin);
    int servoPos = map(potValue, 0, 1023, 0, 180);
    panelServo.write(servoPos);

    // Display manual mode status on the LCD
    lcd.setCursor(0, 0);
    lcd.print("Manual Mode     ");
    lcd.setCursor(0, 1);
    lcd.print("Servo Pos: ");
    lcd.print(servoPos);
  } 
  //--------------------------------------------------------------------------
  //--------------------------------------------------------------------------

  
  
  else if (useProcessing) {
    // Display Processing mode status on the LCD
    lcd.setCursor(0, 0);
    lcd.print("Processing Mode ");
    lcd.setCursor(0, 1);
    lcd.print("Servo Pos: ");
    lcd.print(panelServo.read());
  } 
  
  
  //------------------------------------Automatic Scanning-------------------------------------------
  //-------------------------------------------------------------------------------------------------
  else {
    // Automatic scanning mode
    if (ledOn3 == false) {
      // Initial scan to find the best position
      maxLdrValue = 0;                     // Reset the maximum light intensity value
      for (int pos = 0; pos <= scanRange; pos += stepSize) {
        panelServo.write(pos);           // Move the servo to the current position
        delay(500);                     // Wait for the servo to reach the position and stabilize
        
        ldrValue = analogRead(ldrPin);     // Read the light intensity value
        if (ldrValue > maxLdrValue) {     // Check if this is the highest light intensity found
          maxLdrValue = ldrValue;        // Update the maximum light intensity value
          bestServoPos = pos;           // Updatet he best servo position
        }

        // Print the LDR value and servo position for debugging
        Serial.print("Position: ");
        Serial.print(pos);
        Serial.print("  LDR Value: ");
        Serial.println(ldrValue);

        // Update the LCD with current scanning position and LDR value
        lcd.setCursor(0, 0);
        lcd.print("Scanning Pos: ");
        lcd.print(pos);
        lcd.setCursor(0, 1);
        lcd.print("LDR Value: ");
        lcd.print(ldrValue);
      }

      // Move the panel to the best position found
      panelServo.write(bestServoPos);
      ledOn3 = true;

      // Print the best position for debugging
      Serial.print("Best Position: ");
      Serial.print(bestServoPos);
      Serial.print("  Max LDR Value: ");
      Serial.println(maxLdrValue);

      // Update the LCD with the best position and max LDR value
      lcd.setCursor(0, 0);
      lcd.print("Best Pos: ");
      lcd.print(bestServoPos);
      lcd.setCursor(0, 1);
      lcd.print("Max LDR: ");
      lcd.print(maxLdrValue);

      // Initialize last light intensity and temperature
      lastLightIntensity = maxLdrValue;             // Store the initial light intensity
      lastTemperature = dht.readTemperature();     // Store the initial temperature
    } 
    
    
    
    else {
      // Continuously collect data and check for changes
      float currentTemperature = dht.readTemperature();   // Read the current temperature
      int currentLightIntensity = analogRead(ldrPin);    // Read the current light intensity

      // Display current temperature and light intensity on the LCD
      lcd.setCursor(0, 0);
      lcd.print("Temp: ");
      lcd.print(currentTemperature);
      lcd.print("C      ");
      lcd.setCursor(0, 1);
      lcd.print("Light: ");
      lcd.print(currentLightIntensity);
      lcd.print("      ");
      Serial.print("Temperature = ");
      Serial.print(currentTemperature);
      Serial.print("°C | ");
      Serial.print((currentTemperature*9.0)/5.0+32.0);        // Convert celsius to fahrenheit
      Serial.println("°F ");
 

      // Check if changes exceed the thresholds
      if (abs(currentTemperature - lastTemperature) > tempThreshold &&
          abs(currentLightIntensity - lastLightIntensity) > lightThreshold) {
        lastTemperature = currentTemperature;               // Update the last known temperature
        lastLightIntensity = currentLightIntensity;        // Update the last known light intensity
        ledOn3 = false;                                   // Trigger new scan
      }
      delay(1000);                                      // Delay for 1 second before the next reading
    }
  }
}
